package de.fraunhofer.iem.dataprovider.repository.controller

import de.fraunhofer.iem.dataprovider.configuration.ApiPaths
import de.fraunhofer.iem.dataprovider.configuration.security.JwtContentDto
import de.fraunhofer.iem.dataprovider.configuration.security.JwtService
import de.fraunhofer.iem.dataprovider.gitlab.service.OpenCodeGitlabApi
import de.fraunhofer.iem.dataprovider.kpi.dto.KPITreeResponseDto
import de.fraunhofer.iem.dataprovider.kpi.service.KPIService
import de.fraunhofer.iem.dataprovider.logger.getLogger
import de.fraunhofer.iem.dataprovider.repository.dto.*
import de.fraunhofer.iem.dataprovider.repository.entity.RepositoryEntity
import de.fraunhofer.iem.dataprovider.repository.service.RepositoryService
import de.fraunhofer.iem.dataprovider.toolRun.service.ToolRunService
import de.fraunhofer.iem.dataprovider.user.dto.ValidateUserDto
import de.fraunhofer.iem.dataprovider.user.service.UserService
import org.springframework.http.HttpHeaders
import org.springframework.http.HttpStatus
import org.springframework.http.ResponseCookie
import org.springframework.http.ResponseEntity
import org.springframework.web.bind.annotation.*
import org.springframework.web.server.ResponseStatusException
import java.time.Duration


@RestController
class RepositoryController(
    private val repositoryService: RepositoryService,
    private val kpiService: KPIService,
    private val toolRunService: ToolRunService,
    private val userService: UserService,
    private val jwtService: JwtService,
    private val gitlabApi: OpenCodeGitlabApi
) {

    private val logger = getLogger(javaClass)

    @GetMapping(ApiPaths.REPOSITORY)
    suspend fun getAllRepositories(): List<RepositoryResponseDto> {
        logger.info("Get all repositories")
        return repositoryService.getAllRepositories().map { repositoryEntity: RepositoryEntity ->
            RepositoryResponseDto(
                id = repositoryEntity.id!!,
                name = repositoryEntity.name,
                repositoryId = repositoryEntity.projectId,
                url = repositoryEntity.url
            )
        }
    }

    @GetMapping(ApiPaths.REPOSITORY_SCORE_CARD)
    suspend fun getAllScoreCards(): List<ScoreCardResponseDto> {
        logger.info("Get all repositories as score cards")
        return repositoryService.getAllRepositories().map { repositoryEntity: RepositoryEntity ->
            val rawKpis = repositoryEntity.getLastToolRun().kpiEntities.map { it.toCalculationDto() }
            val rootKpi = kpiService.removeKPIChildrenLowerThanSecondLevel(
                this.kpiService.getKpiTreeForRawKpis(rawKpis).toViewModel()
            )

            ScoreCardResponseDto(
                RepositoryResponseDto(
                    id = repositoryEntity.id!!,
                    name = repositoryEntity.name,
                    repositoryId = repositoryEntity.projectId,
                    url = repositoryEntity.url
                ),
                rootKpi
            )
        }
    }

    @GetMapping(ApiPaths.SCORE_CARD_BY_REPOSITORY_ID)
    suspend fun getScoreCardByRepositoryId(@PathVariable id: Long): ScoreCardResponseDto {
        logger.info("Get repository score card with id $id")
        val repositoryEntity = this.repositoryService.findRepoById(id) ?: throw ResponseStatusException(
            HttpStatus.NOT_FOUND, "repository not found"
        )

        val rawKpis = repositoryEntity.getLastToolRun().kpiEntities.map { it.toCalculationDto() }
        val rootKpi = kpiService.removeKPIChildrenLowerThanSecondLevel(
            this.kpiService.getKpiTreeForRawKpis(rawKpis).toViewModel()
        )

        return ScoreCardResponseDto(
            RepositoryResponseDto(
                id = repositoryEntity.id!!,
                name = repositoryEntity.name,
                repositoryId = repositoryEntity.projectId,
                url = repositoryEntity.url
            ),
            rootKpi
        )
    }

    @GetMapping(ApiPaths.REPOSITORY_ID)
    suspend fun getRepositoryById(@PathVariable id: Long): RepositoryResponseDto {
        logger.info("Get repository with id $id")
        val repositoryEntity = this.repositoryService.findRepoById(id) ?: throw ResponseStatusException(
            HttpStatus.NOT_FOUND, "repository not found"
        )
        return RepositoryResponseDto(
            id = repositoryEntity.id!!,
            name = repositoryEntity.name,
            repositoryId = repositoryEntity.projectId,
            url = repositoryEntity.url
        )
    }

    /**
     * This endpoint is mainly used for debug calls to verify the db contents.
     */
    @GetMapping(ApiPaths.RAW_KPI_BY_REPOSITORY_ID)
    suspend fun getRawKPIByRepositoryId(@PathVariable id: Long): RawKpiDto {
        logger.info("Get raw KPIs for repository id $id")
        val repositoryEntity = this.repositoryService.findRepoById(id) ?: throw ResponseStatusException(
            HttpStatus.NOT_FOUND, "repository not found"
        )
        val rawKpis = repositoryEntity.getLastToolRun().kpiEntities.map {
            val calcDto = it.toCalculationDto()
            calcDto.calculateKPI()
            calcDto
        }
        return RawKpiDto(rawKpis = rawKpis)
    }

    @GetMapping(ApiPaths.KPI_BY_REPOSITORY_ID)
    suspend fun getKPIByRepositoryId(@PathVariable id: Long): KPITreeResponseDto {
        logger.info("Get KPI tree for repository id $id")
        val repositoryEntity = this.repositoryService.findRepoById(id) ?: throw ResponseStatusException(
            HttpStatus.NOT_FOUND, "repository not found"
        )
        val rawKpis = repositoryEntity.getLastToolRun().kpiEntities.map { it.toCalculationDto() }
        val rootKpi = this.kpiService.getKpiTreeForRawKpis(rawKpis)
        return rootKpi.toViewModel()
    }

    @GetMapping(ApiPaths.REPOSITORY_TOOL_RUN)
    suspend fun getToolRunForRepository(
        @CookieValue(name = "gitlabIdToken", required = false) gitlabCookie: String?,
        @PathVariable id: Long
    ): ToolRunResponseDto {
        logger.info("Cookie value $gitlabCookie")

        val isProjectMember = if (gitlabCookie != null) {
            jwtService.getContentIfValid(gitlabCookie)?.let {
                gitlabApi.userIsProjectMember(projectId = id, gitlabUserId = it.gitlabId)
            } ?: false
        } else {
            false
        }

        return ToolRunResponseDto(
            isProjectMember = isProjectMember,
            toolRun = toolRunService.getToolRunForRepository(
                projectId = id,
                includeFindings = isProjectMember
            )
        )
    }

    @PostMapping(ApiPaths.REPOSITORY_UPDATE_CONSENT)
    suspend fun updateRepositoryConsent(@RequestBody repositoryConsentDto: RepositoryConsentDto) {
        repositoryService.updateVisualizationConsent(repositoryConsentDto)
    }

    @PostMapping(ApiPaths.REPOSITORY_VALIDATE_USER)
    suspend fun getGitlabCookie(
        @PathVariable id: Long,
        @RequestBody validateDto: ValidateUserDto
    ): ResponseEntity<OkResponseDto> {
        val gitlabUserId = userService.getGitlabUserId(validateUserDto = validateDto)
        val jwt = jwtService.createJWT(JwtContentDto(gitlabId = gitlabUserId))

        val gitlabCookie = ResponseCookie.from("gitlabIdToken", jwt ?: "")
            .httpOnly(false)
            .secure(true)
            .path("/")
            .maxAge(Duration.ofDays(7))
            .build()

        return ResponseEntity
            .ok()
            .header(HttpHeaders.SET_COOKIE, gitlabCookie.toString())
            .body(OkResponseDto())
    }
}
