package de.fraunhofer.iem.dataprovider.repository.service

import de.fraunhofer.iem.dataprovider.gitlab.service.OpenCodeGitlabApi
import de.fraunhofer.iem.dataprovider.logger.getLogger
import de.fraunhofer.iem.dataprovider.repository.dto.RepositoryConsentDto
import de.fraunhofer.iem.dataprovider.repository.dto.RepositoryCreateDto
import de.fraunhofer.iem.dataprovider.repository.entity.RepositoryEntity
import de.fraunhofer.iem.dataprovider.repository.repository.RepositoryRepository
import org.springframework.stereotype.Service

@Service
class RepositoryService(
    private val repositoryRepository: RepositoryRepository,
    private val openCodeGitlabApi: OpenCodeGitlabApi,
) {
    private val logger = getLogger(javaClass)


    /**
     * Either creates or returns a repository entity based upon its
     * opencode repository id.
     */
    fun getOrCreate(gitRepository: RepositoryCreateDto): RepositoryEntity {
        logger.info("Checking if repository with ${gitRepository.id} exists")
        var repo = repositoryRepository.findByProjectId(gitRepository.id)
        if (repo == null) {
            logger.info("Creating repository with id ${gitRepository.id}")
            repo = repositoryRepository.save(gitRepository.toDbObject())
        }
        return repo
    }

    /**
     * Queries the gitlab api to get the repo name, url, and id.
     */
    fun getRepositoryInfoFromGitlab(projectId: Long): RepositoryCreateDto {
        return openCodeGitlabApi.getRepositoryInfo(projectId)
    }

    fun findRepoById(projectId: Long): RepositoryEntity? {
        return repositoryRepository.findByProjectId(projectId)
    }

    fun getAllRepositories(): List<RepositoryEntity> {
        return repositoryRepository.findAll()
    }

    fun updateVisualizationConsent(repositoryConsentDto: RepositoryConsentDto) {
        val repo = findRepoById(repositoryConsentDto.projectId)
        if (repo != null) {
            repo.visualizationConsent = repositoryConsentDto.consent
            logger.info("Updated visualization consent $repositoryConsentDto")
            repositoryRepository.save(repo)
        } else {
            logger.error(
                "Could not update visualization consent because repo" +
                    "was not found $repositoryConsentDto"
            )
        }
    }

    fun getAllRepositoriesWithConsent(): List<RepositoryEntity> {
        return repositoryRepository.findByVisualizationConsentTrue()
    }

}
